@@ -1,7 +1,9 @@
 ---------------------------------------------------------------------------
+--- Startup notification module for awful
+--
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.5.9
+-- @module awful.startup_notification
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -12,16 +14,18 @@ local capi =
     awesome = awesome,
     root = root
 }
-
---- Startup notification module for awful
--- awful.startup_notification
+local beautiful = require("beautiful")
 
 local app_starting = {}
 
 local cursor_waiting = "watch"
 
+--- Show busy mouse cursor during spawn.
+-- @beautiful beautiful.enable_spawn_cursor
+-- @tparam[opt=true] boolean enable_spawn_cursor
+
 local function update_cursor()
-    if #app_starting > 0 then
+    if #app_starting > 0 and beautiful.enable_spawn_cursor ~= false then
         capi.root.cursor(cursor_waiting)
     else
         capi.root.cursor("left_ptr")
