@@ -22,6 +22,7 @@ local screen = require("awful.screen")
 local util = require("awful.util")
 local gtable = require("gears.table")
 local gfs = require("gears.filesystem")
+local gmath = require("gears.math")
 local beautiful = require("beautiful")
 local wibox = require("wibox")
 local surface = require("gears.surface")
@@ -189,6 +190,18 @@ naughty.notificationClosedReason = {
 -- @beautiful beautiful.notification_height
 -- @tparam int notification_height
 
+--- Notifications maximum width.
+-- @beautiful beautiful.notification_max_width
+-- @tparam int notification_max_width
+
+--- Notifications maximum height.
+-- @beautiful beautiful.notification_max_height
+-- @tparam int notification_max_height
+
+--- Notifications icon size.
+-- @beautiful beautiful.notification_icon_size
+-- @tparam int notification_icon_size
+
 
 -- Counter for the notifications
 -- Required for later access via DBUS
@@ -407,6 +420,12 @@ function naughty.getById(id)
     end
 end
 
+--- Increase notification ID by one
+function naughty.get_next_notification_id()
+  counter = counter + 1
+  return counter
+end
+
 --- Install expiration timer for notification object.
 -- @tparam notification notification Notification object.
 -- @tparam number timeout Time in seconds to be set as expiration timeout.
@@ -490,6 +509,10 @@ local function update_size(notification)
         width = s.actions_max_width
     end
 
+    if s.max_width then
+        width = math.min(width, s.max_width)
+    end
+
     -- calculate the height
     if not height then
         local w = width - (n.iconbox and s.icon_w + 2 * margin or 0) - 2 * margin
@@ -503,6 +526,10 @@ local function update_size(notification)
 
     height = height + s.actions_total_height
 
+    if s.max_height then
+        height = math.min(height, s.max_height)
+    end
+
     -- crop to workarea size if too big
     local workarea = n.screen.workarea
     local border_width = s.border_width or 0
@@ -559,6 +586,8 @@ end
 -- @bool[opt=true] args.ontop Boolean forcing popups to display on top.
 -- @int[opt=`beautiful.notification_height` or auto] args.height Popup height.
 -- @int[opt=`beautiful.notification_width` or auto] args.width Popup width.
+-- @int[opt=`beautiful.notification_max_height` or auto] args.max_height Popup maximum height.
+-- @int[opt=`beautiful.notification_max_width` or auto] args.max_width Popup maximum width.
 -- @string[opt=`beautiful.notification_font` or `beautiful.font` or `awesome.font`] args.font Notification font.
 -- @string[opt] args.icon Path to icon.
 -- @int[opt] args.icon_size Desired icon size in px.
@@ -600,7 +629,8 @@ function naughty.notify(args)
         args.preset or naughty.config.presets.normal or {})
     local timeout = args.timeout or preset.timeout
     local icon = args.icon or preset.icon
-    local icon_size = args.icon_size or preset.icon_size
+    local icon_size = args.icon_size or preset.icon_size or
+        beautiful.notification_icon_size
     local text = args.text or preset.text
     local title = args.title or preset.title
     local s = get_screen(args.screen or preset.screen or screen.focused())
@@ -633,6 +663,10 @@ function naughty.notify(args)
         beautiful.notification_width
     local height = args.height or preset.height or
         beautiful.notification_height
+    local max_width = args.max_width or preset.max_width or
+        beautiful.notification_max_width
+    local max_height = args.max_height or preset.max_height or
+        beautiful.notification_max_height
     local margin = args.margin or preset.margin or
         beautiful.notification_margin
     local opacity = args.opacity or preset.opacity or
@@ -753,10 +787,15 @@ function naughty.notify(args)
         if icon then
             iconbox = wibox.widget.imagebox()
             iconmargin = wibox.container.margin(iconbox, margin, margin, margin, margin)
-            if icon_size then
-                local scaled = cairo.ImageSurface(cairo.Format.ARGB32, icon_size, icon_size)
+            if icon_size and (icon:get_height() > icon_size or icon:get_width() > icon_size) then
+                local scale_factor = icon_size / math.max(icon:get_height(),
+                                                          icon:get_width())
+                local scaled =
+                    cairo.ImageSurface(cairo.Format.ARGB32,
+                                       gmath.round(icon:get_width() * scale_factor),
+                                       gmath.round(icon:get_height() * scale_factor))
                 local cr = cairo.Context(scaled)
-                cr:scale(icon_size / icon:get_height(), icon_size / icon:get_width())
+                cr:scale(scale_factor, scale_factor)
                 cr:set_source_surface(icon, 0, 0)
                 cr:paint()
                 icon = scaled
@@ -770,24 +809,26 @@ function naughty.notify(args)
     notification.iconbox = iconbox
 
     -- create container wibox
-    notification.box = wibox({ fg = fg,
-                               bg = bg,
-                               border_color = border_color,
-                               border_width = border_width,
-                               shape_border_color = shape and border_color,
-                               shape_border_width = shape and border_width,
-                               shape = shape,
-                               type = "notification" })
-
-    if reuse_box then
+    if not reuse_box then
+        notification.box = wibox({ type = "notification" })
+    else
         notification.box = reuse_box
     end
+    notification.box.fg = fg
+    notification.box.bg = bg
+    notification.box.border_color = border_color
+    notification.box.border_width = border_width
+    notification.box.shape_border_color = shape and border_color
+    notification.box.shape_border_width = shape and border_width
+    notification.box.shape = shape
 
     if hover_timeout then notification.box:connect_signal("mouse::enter", hover_destroy) end
 
     notification.size_info = {
         width = width,
         height = height,
+        max_width = max_width,
+        max_height = max_height,
         icon_w = icon_w,
         icon_h = icon_h,
         margin = margin,
