@@ -63,6 +63,37 @@ function wibox:find_widgets(x, y)
     return self._drawable:find_widgets(x, y)
 end
 
+--- Create a widget that reflects the current state of this wibox.
+-- @treturn widget A new widget.
+function wibox:to_widget()
+    local bw = self.border_width or beautiful.border_width or 0
+    return wibox.widget {
+        {
+            self:get_widget(),
+            margins = bw,
+            widget  = wibox.container.margin
+        },
+        bg                 = self.bg or beautiful.bg_normal or "#ffffff",
+        fg                 = self.fg or beautiful.fg_normal or "#000000",
+        shape_border_color = self.border_color or beautiful.border_color or "#000000",
+        shape_border_width = bw*2,
+        shape_clip         = true,
+        shape              = self._shape,
+        forced_width       = self:geometry().width  + 2*bw,
+        forced_height      = self:geometry().height + 2*bw,
+        widget             = wibox.container.background
+    }
+end
+
+--- Save a screenshot of the wibox to `path`.
+-- @tparam string path The path.
+-- @tparam[opt=nil] table context A widget context.
+function wibox:save_to_svg(path, context)
+    wibox.widget.draw_to_svg_file(
+        self:to_widget(), path, self:geometry().width, self:geometry().height, context
+    )
+end
+
 function wibox:_apply_shape()
     local shape = self._shape
 
@@ -106,11 +137,6 @@ function wibox:_apply_shape()
     img:finish()
 end
 
---- Set the wibox shape.
--- @property shape
--- @tparam gears.shape A gears.shape compatible function.
--- @see gears.shape
-
 function wibox:set_shape(shape)
     self._shape = shape
     self:_apply_shape()
@@ -120,6 +146,24 @@ function wibox:get_shape()
     return self._shape
 end
 
+function wibox:set_input_passthrough(value)
+    rawset(self, "_input_passthrough", value)
+
+    if not value then
+        self.shape_input = nil
+    else
+        local img = cairo.ImageSurface(cairo.Format.A1, 0, 0)
+        self.shape_input = img._native
+        img:finish()
+    end
+
+    self:emit_signal("property::input_passthrough", value)
+end
+
+function wibox:get_input_passthrough()
+    return self._input_passthrough
+end
+
 function wibox:get_screen()
     if self.screen_assigned and self.screen_assigned.valid then
         return self.screen_assigned
@@ -148,6 +192,19 @@ function wibox:set_screen(s)
     self._drawable:_force_screen(s)
 end
 
+function wibox:get_children_by_id(name)
+    --TODO v5: Move the ID management to the hierarchy.
+    if rawget(self, "_by_id") then
+        return rawget(self, "_by_id")[name]
+    elseif self._drawable.widget
+      and self._drawable.widget._private
+      and self._drawable.widget._private.by_id then
+          return self._drawable.widget._private.by_id[name]
+    end
+
+    return {}
+end
+
 for _, k in pairs{ "buttons", "struts", "geometry", "get_xproperty", "set_xproperty" } do
     wibox[k] = function(self, ...)
         return self.drawin[k](self.drawin, ...)
@@ -192,25 +249,7 @@ end
 
 --- Create a wibox.
 -- @tparam[opt=nil] table args
--- @tparam integer args.border_width Border width.
--- @tparam string args.border_color Border color.
--- @tparam boolean args.ontop On top of other windows.
--- @tparam string args.cursor The mouse cursor.
--- @tparam boolean args.visible Visibility.
--- @tparam number args.opacity The opacity of the wibox, between 0 and 1.
--- @tparam string args.type The window type (desktop, normal, dock, …).
--- @tparam integer args.x The x coordinates.
--- @tparam integer args.y The y coordinates.
--- @tparam integer args.width The width of the wibox.
--- @tparam integer args.height The height of the wibox.
--- @tparam screen args.screen The wibox screen.
--- @tparam wibox.widget args.widget The widget that the wibox displays.
--- @param args.shape_bounding The wibox’s bounding shape as a (native) cairo surface.
--- @param args.shape_clip The wibox’s clip shape as a (native) cairo surface.
--- @param args.shape_input The wibox’s input shape as a (native) cairo surface.
--- @tparam color args.bg The background of the wibox.
--- @tparam surface args.bgimage The background image of the drawable.
--- @tparam color args.fg The foreground (text) of the wibox.
+--
 -- @treturn wibox The new wibox
 -- @function .wibox
 
@@ -227,6 +266,10 @@ local function new(args)
     ret._drawable = wibox.drawable(w.drawable, { wibox = ret },
         "wibox drawable (" .. object.modulename(3) .. ")")
 
+    function ret._drawable.get_wibox()
+        return ret
+    end
+
     ret._drawable:_inform_visible(w.visible)
     w:connect_signal("property::visible", function()
         ret._drawable:_inform_visible(w.visible)
@@ -272,7 +315,7 @@ local function new(args)
         __newindex = function(self, k,v)
             if rawget(self, "set_"..k) then
                 self["set_"..k](self, v)
-            elseif w[k] ~= nil or force_forward[k] then
+            elseif force_forward[k] or w[k] ~= nil then
                 w[k] = v
             else
                 rawset(self, k, v)
@@ -293,7 +336,13 @@ local function new(args)
         ret:set_screen ( args.screen  )
     end
 
-    ret.shape = args.shape
+    if args.shape then
+        ret.shape = args.shape
+    end
+
+    if args.screen then
+        ret.input_passthrough = args.input_passthrough
+    end
 
     return ret
 end
