@@ -91,7 +91,7 @@ function imagebox:set_image(image)
         end
     end
 
-    image = surface.load(image)
+    image = image and surface.load(image)
 
     if image then
         local w = image.width
@@ -104,7 +104,7 @@ function imagebox:set_image(image)
     if self._private.image == image then
         -- The image could have been modified, so better redraw
         self:emit_signal("widget::redraw_needed")
-        return
+        return true
     end
 
     self._private.image = image
